<?php

namespace Vote\Vote\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Vote\Vote\Http\Requests\VoteRequest;
use Vote\Vote\Interfaces\VoteRepositoryInterface;
use Vote\Vote\Models\Vote;

/**
 * Resource controller class for vote.
 */
class VoteResourceController extends BaseController
{

    /**
     * Initialize vote resource controller.
     *
     * @param type VoteRepositoryInterface $vote
     *
     * @return null
     */
    public function __construct(VoteRepositoryInterface $vote)
    {
        parent::__construct();
        $this->repository = $vote;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Vote\Vote\Repositories\Criteria\VoteResourceCriteria::class);
    }

    /**
     * Display a list of vote.
     *
     * @return Response
     */
    public function index(VoteRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Vote\Vote\Repositories\Presenter\VotePresenter::class)
                ->$function();
        }

        $votes = $this->repository->paginate();

        return $this->response->title(trans('vote::vote.names'))
            ->view('vote::vote.index', true)
            ->data(compact('votes', 'view'))
            ->output();
    }

    /**
     * Display vote.
     *
     * @param Request $request
     * @param Model   $vote
     *
     * @return Response
     */
    public function show(VoteRequest $request, Vote $vote)
    {

        if ($vote->exists) {
            $view = 'vote::vote.show';
        } else {
            $view = 'vote::vote.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('vote::vote.name'))
            ->data(compact('vote'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new vote.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(VoteRequest $request)
    {

        $vote = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('vote::vote.name')) 
            ->view('vote::vote.create', true) 
            ->data(compact('vote'))
            ->output();
    }

    /**
     * Create new vote.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(VoteRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $vote                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('vote::vote.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('vote/vote/' . $vote->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/vote/vote'))
                ->redirect();
        }

    }

    /**
     * Show vote for editing.
     *
     * @param Request $request
     * @param Model   $vote
     *
     * @return Response
     */
    public function edit(VoteRequest $request, Vote $vote)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('vote::vote.name'))
            ->view('vote::vote.edit', true)
            ->data(compact('vote'))
            ->output();
    }

    /**
     * Update the vote.
     *
     * @param Request $request
     * @param Model   $vote
     *
     * @return Response
     */
    public function update(VoteRequest $request, Vote $vote)
    {
        try {
            $attributes = $request->all();

            $vote->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('vote::vote.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('vote/vote/' . $vote->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('vote/vote/' . $vote->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the vote.
     *
     * @param Model   $vote
     *
     * @return Response
     */
    public function destroy(VoteRequest $request, Vote $vote)
    {
        try {

            $vote->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('vote::vote.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('vote/vote/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('vote/vote/' . $vote->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple vote.
     *
     * @param Model   $vote
     *
     * @return Response
     */
    public function delete(VoteRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('vote::vote.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('vote/vote'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/vote/vote'))
                ->redirect();
        }

    }

    /**
     * Restore deleted votes.
     *
     * @param Model   $vote
     *
     * @return Response
     */
    public function restore(VoteRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('vote::vote.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/vote/vote'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/vote/vote/'))
                ->redirect();
        }

    }

}
